#!/usr/bin/ruby

require 'test/unit'
require 'symbiosis/crontab'
require 'pp'

class TestCrontab < Test::Unit::TestCase

  def setup
    @crontab = nil
  end

  def teardown
  end

  def test_documentation_example

    assert_nothing_raised do
      # Note this crontab has no newline at the end.  Specially for Mr Howells.
      @crontab = Symbiosis::Crontab.new " 
#
# Send any output to Bob
#
MAILTO=bob@my-brilliant-site.com

#
# run at 18:40 every day
#
40 18 * * *       echo Hello Dave.

#
# run at 9am every Monday - Friday
#
0   9 * * mon-fri wget http://www.my-brilliant-site.com/cron.php

#
# Run once a month
#
@monthly          /usr/local/bin/monthly-job.sh

#
# run every minute
#
* * * * * php --some-argument-or-other cronjob.php
"
    end

    assert_equal("bob@my-brilliant-site.com",@crontab.environment["MAILTO"])
    
    # Test first entry -- run at 18:40 every day
    #
    record = @crontab.records[0]
    assert_equal([40], record.min)
    assert_equal([18], record.hour)
    assert_equal((1..31).to_a, record.mday)
    assert_equal((1..12).to_a, record.mon)
    assert_equal((0..6).to_a, record.wday)
    assert_equal("echo Hello Dave.", record.command)

    # Next entry -- run at 9am every Monday - Friday
    #
    record = @crontab.records[1]
    assert_equal([0], record.min)
    assert_equal([9], record.hour)
    assert_equal((1..31).to_a, record.mday)
    assert_equal((1..12).to_a, record.mon)
    assert_equal((1..5).to_a, record.wday)
    assert_equal("wget http://www.my-brilliant-site.com/cron.php", record.command)

    # Next entry -- Run once a month
    #
    record = @crontab.records[2]
    assert_equal([0], record.min)
    assert_equal([0], record.hour)
    assert_equal([1], record.mday)
    assert_equal((1..12).to_a, record.mon)
    assert_equal((0..6).to_a, record.wday)
    assert_equal("/usr/local/bin/monthly-job.sh", record.command)
    
    # Next entry -- Run every minute
    #
    record = @crontab.records[3]
    assert_equal((0..59).to_a, record.min)
    assert_equal((0..23).to_a, record.hour)
    assert_equal((1..31).to_a, record.mday)
    assert_equal((1..12).to_a, record.mon)
    assert_equal((0..6).to_a, record.wday)
    assert_equal("php --some-argument-or-other cronjob.php", record.command)
  end

end


class TestCrontabRecord < Test::Unit::TestCase

  def setup
    @crontab_record = nil
  end

  def teardown
  end

  def test_manpage_eg1
    # run five minutes after midnight, every day
    assert_nothing_raised {
      @crontab_record = Symbiosis::CrontabRecord.parse("5 0 * * *       $HOME/bin/daily.job >> $HOME/tmp/out 2>&1")
    }
    assert_equal([5], @crontab_record.min)
    assert_equal([0], @crontab_record.hour)
    assert_equal((1..31).to_a, @crontab_record.mday)
    assert_equal((1..12).to_a, @crontab_record.mon)
    assert_equal((0..6).to_a, @crontab_record.wday)
    assert_equal("$HOME/bin/daily.job >> $HOME/tmp/out 2>&1", @crontab_record.command)
  end

  def test_manpage_eg2
    # run at 2:15pm on the first of every month
    assert_nothing_raised {
      @crontab_record = Symbiosis::CrontabRecord.parse("15 14 1 * *     $HOME/bin/monthly")
    }

    assert_equal([15], @crontab_record.min)
    assert_equal([14], @crontab_record.hour)
    assert_equal([1], @crontab_record.mday)
    assert_equal((1..12).to_a, @crontab_record.mon)
    assert_equal((0..6).to_a, @crontab_record.wday)
    assert_equal("$HOME/bin/monthly", @crontab_record.command)
  end

  def test_manpage_eg3
    # run at 10 pm on weekdays, annoy Joe
    assert_nothing_raised {
      @crontab_record = Symbiosis::CrontabRecord.parse("0 22 * * 1-5    mail -s \"It's 10pm\" joe%Joe,%%Where are your kids?%")
    }

    assert_equal([0], @crontab_record.min)
    assert_equal([22], @crontab_record.hour)
    assert_equal((1..31).to_a, @crontab_record.mday)
    assert_equal((1..12).to_a, @crontab_record.mon)
    assert_equal((1..5).to_a, @crontab_record.wday)
    assert_equal("mail -s \"It's 10pm\" joe%Joe,%%Where are your kids?%", @crontab_record.command)
    
  end

  def test_manpage_eg4 
#       23 0-23/2 * * * echo "run 23 minutes after midn, 2am, 4am ..., everyday"
    assert_nothing_raised {
      @crontab_record = Symbiosis::CrontabRecord.parse('23 0-23/2 * * * echo "run 23 minutes after midn, 2am, 4am ..., everyday"')
    }
    assert_equal([23], @crontab_record.min)
    assert_equal([0,2,4,6,8,10,12,14,16,18,20,22], @crontab_record.hour)
    assert_equal((1..31).to_a, @crontab_record.mday)
    assert_equal((1..12).to_a, @crontab_record.mon)
    assert_equal((0..6).to_a, @crontab_record.wday)
    assert_equal('echo "run 23 minutes after midn, 2am, 4am ..., everyday"', @crontab_record.command)
  end

  def test_manpage_eg5
    assert_nothing_raised {
      @crontab_record = Symbiosis::CrontabRecord.parse('5 4 * * sun     echo "run at 5 after 4 every sunday"')
    }
    assert_equal([5], @crontab_record.min)
    assert_equal([4], @crontab_record.hour)
    assert_equal((1..31).to_a, @crontab_record.mday)
    assert_equal((1..12).to_a, @crontab_record.mon)
    assert_equal([0], @crontab_record.wday)
    assert_equal('echo "run at 5 after 4 every sunday"', @crontab_record.command)
  end

  def test_hourly
    assert_nothing_raised {
      @crontab_record = Symbiosis::CrontabRecord.parse("@hourly do my stuff")
    }
    assert_equal([0], @crontab_record.min)
    assert_equal((0..23).to_a, @crontab_record.hour)
    assert_equal((1..31).to_a, @crontab_record.mday)
    assert_equal((1..12).to_a, @crontab_record.mon)
    assert_equal((0..6).to_a, @crontab_record.wday)
    assert_equal("do my stuff", @crontab_record.command)
  end
  
  def test_daily_midnight
    @crontab_record = nil
    %w(daily midnight).each do |d|
      assert_nothing_raised {
        @crontab_record = Symbiosis::CrontabRecord.parse("@#{d} do my stuff")
      }
      assert_equal([0], @crontab_record.min)
      assert_equal([0], @crontab_record.hour)
      assert_equal((1..31).to_a, @crontab_record.mday)
      assert_equal((1..12).to_a, @crontab_record.mon)
      assert_equal((0..6).to_a, @crontab_record.wday)
      assert_equal("do my stuff", @crontab_record.command)
    end
  end


  def test_weekly
    @crontab_record = Symbiosis::CrontabRecord.parse("@weekly do my stuff")
    assert_equal([0], @crontab_record.min)
    assert_equal([0], @crontab_record.hour)
    assert_equal((1..31).to_a, @crontab_record.mday)
    assert_equal((1..12).to_a, @crontab_record.mon)
    assert_equal([0], @crontab_record.wday)
    assert_equal("do my stuff", @crontab_record.command)
  end

  
  def test_monthly
    @crontab_record = Symbiosis::CrontabRecord.parse("@monthly do my stuff")
    assert_equal([0], @crontab_record.min)
    assert_equal([0], @crontab_record.hour)
    assert_equal([1], @crontab_record.mday)
    assert_equal((1..12).to_a, @crontab_record.mon)
    assert_equal((0..6).to_a, @crontab_record.wday)
    assert_equal("do my stuff", @crontab_record.command)
  end

  def test_monthly
    %w(annually yearly).each do |d|
      @crontab_record = Symbiosis::CrontabRecord.parse("@#{d} do my stuff")
      assert_equal([0], @crontab_record.min)
      assert_equal([0], @crontab_record.hour)
      assert_equal([1], @crontab_record.mday)
      assert_equal([1], @crontab_record.mon)
      assert_equal((0..6).to_a, @crontab_record.wday)
      assert_equal("do my stuff", @crontab_record.command)
    end
  end

  def test_wday_names
    names = %w(sun mon tue wed thu fri sat sun)
    values = [0,1,2,3,4,5,6,0]
    names.each_with_index do |wday,ind|
      @crontab_record = Symbiosis::CrontabRecord.parse("* * * * #{wday} do my bidding")
      assert_equal([values[ind]], @crontab_record.wday)
    end

    @crontab_record = Symbiosis::CrontabRecord.parse("* * * * mon-fri do my bidding")
    assert_equal((1..5).to_a, @crontab_record.wday)

    @crontab_record = Symbiosis::CrontabRecord.parse("* * * * sat-sun do my bidding")
    assert_equal([0,6], @crontab_record.wday)

    # Stupid people not reading instructions.
    @crontab_record = Symbiosis::CrontabRecord.parse("* * * * sUnDaY-SATURDAY do my bidding")
    assert_equal((0..6).to_a, @crontab_record.wday)
  end

  def test_mon_names
    names = %w(jan feb mar apr may jun jul aug sep oct nov dec)
    values =   [1, 2,  3,  4,  5,  6,  7,  8,  9,  10, 11, 12]
    names.each_with_index do |mon,ind|
      @crontab_record = Symbiosis::CrontabRecord.parse("* * * #{mon} * do my bidding")
      assert_equal([values[ind]], @crontab_record.mon)
    end

    # a range of names
    @crontab_record = Symbiosis::CrontabRecord.parse("* * * jan,mar,jun-sep * do my bidding")
    assert_equal([1,3,6,7,8,9], @crontab_record.mon)

    # test long names
    @crontab_record = Symbiosis::CrontabRecord.parse("* * * january,february,march,april,may,june,july,august,september,october,november,december * do my bidding")
    assert_equal((1..12).to_a, @crontab_record.mon)
   
    # make sure that the regexp starts from the beginning of a word.
    @crontab_record = Symbiosis::CrontabRecord.parse("* * * janjan,marfeb * do my bidding")
    assert_equal([1,3], @crontab_record.mon)

    # Test year boundary parsing
    @crontab_record = Symbiosis::CrontabRecord.parse("* * * nov-feb * do my bidding")
    assert_equal([1,2,11,12], @crontab_record.mon)
  end

end
